/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
/*
 *  $RCSfile: $
 *  $Revision: $  $Date: $ 
 */
/**
 * 
 */
package com.ibm.ram.internal.batch.filesystem;

import java.text.MessageFormat;
import java.util.*;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;

import com.ibm.ram.common.data.Asset;
import com.ibm.ram.internal.rich.ui.util.Messages;

/**
 * An abstract base class for rule containers. This class takes care of the storage
 * and retrieval of child rules.
 * 
 * @author ebordeau
 * @since 7.1
 */
public abstract class AbstractRuleContainer extends AbstractRule implements IRuleContainer {
	
	private List children = new ArrayList();

	public void add(IRule rule) {
		rule.setParent(this);
		children.add(rule);
	}

	public IRule[] getRules() {
		return (IRule[])children.toArray(new IRule[children.size()]);
	}

	public void remove(IRule rule) {
		children.remove(rule);
	}
	
	public void initialize(Map config) {
		super.initialize(config);
		Map[] ruleMaps = (Map[])config.get(RULES_KEY);
		for (int i = 0; i < ruleMaps.length; i++) {
			String ruleId = (String)ruleMaps[i].get(RULE_ID_KEY);
			IRule rule = RuleFactory.createRule(ruleId);
			add(rule);
			rule.initialize(ruleMaps[i]);
		}
	}

	public void save(Map config) {
		super.save(config);
		Map[] ruleMaps = new HashMap[getRules().length];
		IRule[] children = getRules();
		for (int i = 0; i < children.length; i++) {
			ruleMaps[i] = new HashMap();
			children[i].save(ruleMaps[i]);
		}
		config.put(RULES_KEY, ruleMaps);
	}

	public void apply(Object object, Asset asset, IProgressMonitor monitor) {
		super.apply(object, asset, monitor);
		IRule[] rules = getRules();
		for (int i = 0; i < rules.length && !monitor.isCanceled(); i++) {
			IRule rule = rules[i];
			if (!rule.applyValue(asset)) {
				Object[] children = rule.filter(object);
				if (children != null) {
					SubProgressMonitor childMon = new SubProgressMonitor(monitor, 100);
					childMon.beginTask(Messages.FileSystemDataSource_APPLY_FILE_SYSTEM_CHILD_RULES, children.length * 100);
					for (int j = 0; j < children.length && !monitor.isCanceled(); j++) {
						Object child = children[j];
						monitor.subTask(MessageFormat.format(Messages.FileSystemDataSource_APPLY_RULE_ON_CHILD, new Object[] {rule, child}));
						rule.apply(child, asset, childMon);
						childMon.worked(100);
					}
				}
			}
		}
	}
	
	public List apply(Object object, IProgressMonitor monitor) {
		List result = super.apply(object, monitor);
		if (result == null)
			result = new ArrayList();
		Asset asset = null;
		IRule[] rules = getRules();
		for (int i = 0; i < rules.length && !monitor.isCanceled(); i++) {
			IRule rule = rules[i];
			Object[] children = rule.filter(object);
			if (children != null) {
				SubProgressMonitor childMon = new SubProgressMonitor(monitor, 100);
				childMon.beginTask(Messages.FileSystemDataSource_APPLY_FILE_SYSTEM_CHILD_RULES, children.length * 100);
				for (int j = 0; j < children.length && !monitor.isCanceled(); j++) {
					Object child = children[j];
					monitor.subTask(MessageFormat.format(Messages.FileSystemDataSource_APPLY_RULE_ON_CHILD, new Object[] {rule, child}));
					if (asset != null) {
						rule.apply(child, asset, childMon);
					} else {
						List assets = rule.apply(child, childMon);
						if (assets != null) {
							Iterator iter = assets.iterator();
							while (iter.hasNext() && !monitor.isCanceled()) {
								Asset a = (Asset)iter.next();
								super.apply(object, a, childMon);
							}
							result.addAll(assets);
						}
					}
					childMon.worked(100);
				}
			}
		}
		return result;
	}
	
}
