/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
/*
 *  $RCSfile: $
 *  $Revision: $  $Date: $ 
 */

package com.ibm.ram.internal.batch.filesystem;

import java.io.File;
import java.text.MessageFormat;
import java.util.*;

import org.eclipse.core.runtime.*;

import com.ibm.ram.client.batch.BatchDataSource;
import com.ibm.ram.client.status.RAMStatus;
import com.ibm.ram.client.status.RAMStatusMonitor;
import com.ibm.ram.common.data.Asset;
import com.ibm.ram.internal.common.util.Utilities;
import com.ibm.ram.internal.rich.ui.util.Messages;

/**
 * 
 * @author ebordeau
 * @since 7.1
 */
public class FileSystemDataSource extends BatchDataSource implements IRuleContainer {

	private static final String ROOT_KEY = "root"; //$NON-NLS-1$
	public static final String ID = "com.ibm.ram.batch.filesystem"; //$NON-NLS-1$

	private File root;
	private String rootPath;
	private List rules = new ArrayList();
	private Map assets = new HashMap();

	public FileSystemDataSource() {}

	public FileSystemDataSource(String path) {
		rootPath = path;
	}

	public void add(IRule rule) {
		rule.setParent(this);
		rules.add(rule);
	}

	public Asset get(String key) {
		return (Asset) assets.get(key);
	}

	public void put(String path, Asset asset) {
		assets.put(path, asset);
	}

	public Asset[] fetchAssets(RAMStatusMonitor monitor) {
		if (root == null) {
			if (rootPath == null)
				return new Asset[0];
			root = new File(rootPath);
		}
		List result = apply(root, monitor);
		return (Asset[]) result.toArray(new Asset[result.size()]);
	}

	private List apply(File file, RAMStatusMonitor monitor) {
		IRule[] rules = getRules();
		List result = new ArrayList();

		if (monitor == null)
			monitor = new RAMStatusMonitor(new NullProgressMonitor(), 10);
		try {
			if (rules.length > 0) {
				monitor.beginTask(Messages.FileSystemDataSource_APPLY_FILE_SYSTEM_RULES, rules.length * 100);
				for (int i = 0; !monitor.isCanceled() && i < rules.length; i++) {
					IRule rule = rules[i];
					monitor.subTask(MessageFormat.format(Messages.FileSystemDataSource_APPLY_RULE, new Object[] {rule}));

					Object[] children = rule.filter(file);
					if (children != null) {
						SubProgressMonitor childMon = new SubProgressMonitor(monitor, 90);
						childMon.beginTask(Messages.FileSystemDataSource_APPLY_FILE_SYSTEM_CHILD_RULES, children.length * 100);
						for (int j = 0; !monitor.isCanceled() && j < children.length; j++) {
							Object child = children[j];
							monitor.subTask(MessageFormat.format(Messages.FileSystemDataSource_APPLY_RULE_ON_CHILD, new Object[] {rule, child}));
							List l = rule.apply(child, childMon);
							if (l != null){
								result.addAll(l);
								
								for(int k = 0; k < l.size(); k++){
									monitor.appendStatus(l.get(k), RAMStatus.INFO, RAMStatus.ADD_SUCCESSFUL,
											MessageFormat.format(Messages.BatchUpdatePage_ADD_ASSET, new Object[] {l.get(k)}), null);
								}
							}
							childMon.worked(10);
						}
					} else {
						monitor.worked(100);
					}
				}
			} else {
				monitor.beginTask(Messages.FileSystemDataSource_APPLY_FILE_SYSTEM_RULES, rules.length * 100);
				monitor.worked(100);
			}
		} finally{
			monitor.done();
		}
		return result;
	}

	public Object[] getChildren(Object object) {
		if (object instanceof File) {
			File file = (File) object;
			return file.listFiles();
		}
		return new Object[0];
	}

	public String getRootPath() {
		return rootPath;
	}

	public IRule[] getRules() {
		return (IRule[]) rules.toArray(new IRule[rules.size()]);
	}

	public void remove(IRule rule) {
		rules.remove(rule);
	}

	public void initialize(String metadata) {
		HashMap config = Utilities.decodeXML(metadata);
		rootPath = (String) config.get(ROOT_KEY);
		Map[] ruleMaps = (Map[]) config.get(IRule.RULES_KEY);
		for (int i = 0; i < ruleMaps.length; i++) {
			String className = (String) ruleMaps[i].get(IRule.RULE_ID_KEY);
			IRule rule = RuleFactory.createRule(className);
			add(rule);
			rule.initialize(ruleMaps[i]);
		}
	}

	public String save() {
		HashMap config = new HashMap();
		config.put(ROOT_KEY, getRootPath());
		Map[] ruleMaps = new HashMap[getRules().length];
		IRule[] children = getRules();
		for (int i = 0; i < children.length; i++) {
			ruleMaps[i] = new HashMap();
			children[i].save(ruleMaps[i]);
		}
		config.put(IRule.RULES_KEY, ruleMaps);
		return Utilities.encodeToXML(config);
	}

	public void setRootPath(String path) {
		root = null;
		rootPath = path;
	}

	public String getTypeId() {
		return ID;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		if(rootPath == null)
			return super.toString();
		else
			return "FileSystemDataSource[" + rootPath + "]";  //$NON-NLS-1$//$NON-NLS-2$
	}	

	
}
