/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
/*
 *  $RCSfile: $
 *  $Revision: $  $Date: $ 
 */
/**
 * 
 */
package com.ibm.ram.internal.batch.filesystem;

import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.ram.common.data.Asset;

/**
 * Represents a rule used to create RAM objects from the local filesystem.
 * 
 * @author ebordeau
 * @since 7.1
 */
public interface IRule {
	
	static final String RULES_KEY = "rules"; //$NON-NLS-1$
	static final String FILTER_KEY = "filter"; //$NON-NLS-1$
	static final String RULE_ID_KEY = "ruleId"; //$NON-NLS-1$

	/**
	 * Apply this rule to the given object. If assets are created by this rule,
	 * return the list of assets.
	 * 
	 * @param object the object used to apply this rule
	 * @return a list of assets created by this rule, or null if no assets were created
	 */
	List apply(Object object, IProgressMonitor monitor);
	
	/**
	 * Apply this rule to the given object. An asset has already been created and passed
	 * in as a parameter to this rule.
	 * 
	 * @param object the object used to apply this rule
	 * @param asset the asset
	 */
	void apply(Object object, Asset asset, IProgressMonitor monitor);
	
	/**
	 * Apply this rule's static value to the asset.
	 * 
	 * @param asset the asset
	 * @return true if the value was applied
	 */
	boolean applyValue(Asset asset);
	
	/**
	 * Filter and return the children of the given object.
	 * 
	 * @param parent the parent object
	 * @return the filtered children of the parent object
	 */
	Object[] filter(Object parent);
	
	/**
	 * Returns an array of Strings used as patterns that specify what files to include. 
	 * Used by Ant's DirectoryScanner.
	 * 
	 * @return the include patterns
	 */
	String getFilter();
	
	String getId();
	
	/**
	 * Returns the static user-defined value for this rule.
	 * 
	 * @return the value
	 */
	String getValue();
	
	IRuleContainer getRoot();
	
	void initialize(Map config);
	
	void save(Map config);
	
	/**
	 * Sets the patterns used to specify what files to include in this rule. Used by Ant's 
	 * DirectoryScanner.
	 * 
	 * @param includes a space-separated list of file patterns
	 */
	void setFilter(String filter);
	
	/**
	 * @return this rule's parent
	 */
	IRuleContainer getParent();
	
	/**
	 * Sets this rule's parent.
	 * 
	 * @param parent the parent rule
	 */
	void setParent(IRuleContainer parent);
	
	/**
	 * Sets the static user-defined value for this rule.
	 * 
	 * @param value the value
	 */
	void setValue(String value);

}
