/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
/*
 *  $RCSfile: $
 *  $Revision: $  $Date: $ 
 */
package com.ibm.ram.internal.batch.filesystem;

import java.io.*;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.XMIResource;

import com.ibm.ram.client.LocalFileArtifact;
import com.ibm.ram.common.data.*;
import com.ibm.ram.common.data.Artifact;
import com.ibm.ram.common.data.Asset;
import com.ibm.ram.common.util.*;
import com.ibm.ram.common.util.ManifestAccessor.ClassificationInfo;
import com.ibm.ram.defaultprofile.*;
import com.ibm.ram.defaultprofile.util.Utilities;
import com.ibm.ram.internal.client.util.AssetConstants;
import com.ibm.ram.internal.rich.ui.util.Messages;

public class ManifestUtil {
	
	private static final Logger logger = Logger.getLogger(ManifestUtil.class.getName());

	public static ManifestAccessor loadManifest(File manifestFile) {
		FileInputStream in = null;
		try {
			com.ibm.ram.defaultprofile.Asset asset = null;
			in = new FileInputStream(manifestFile);
			if (in.available() != 0) {
				CachedRAMURIConverter c = new CachedRAMURIConverter(null);
				ResourceSet resSet = ManifestBuilder.createResourceSet(c);
				Resource manifestRes = resSet.createResource(URI.createURI(manifestFile.toURI().toString()));
				((XMIResource)manifestRes).setEncoding("UTF-8"); //$NON-NLS-1$
				manifestRes.load(in, null);
				if (manifestRes.getContents() != null) {
					asset = (com.ibm.ram.defaultprofile.Asset)manifestRes.getContents().get(0);
					ManifestBuilder.setServerPath(asset.eResource().getResourceSet(), ""); //$NON-NLS-1$
					return new ManifestAccessor(asset, null, null);
				}
			}
		} catch (FileNotFoundException e) {
			logger.log(Level.SEVERE, Messages.DownloadAssetHandler_ErrorProcessingManifest, e);
		} catch (IOException e) {
			logger.log(Level.SEVERE, Messages.DownloadAssetHandler_ErrorProcessingManifest, e);
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {}
			}
		}
		return null;
	}
	
	private static void addSubCategory(Asset asset, String fullPath) {
		String[] tokens = fullPath.split(AssetConstants.FILE_DELIM);
		if (tokens.length > 2) { // must have schema, category and at least one subcategory
			String schemaName = tokens[0];
			String categoryName = tokens[1];
			String[] subCategories = new String[tokens.length - 2];
			System.arraycopy(tokens, 2, subCategories, 0, subCategories.length);
			addSubCategory(asset, schemaName, categoryName, subCategories);
		}
	}

	private static SubCategory addSubCategory(Asset asset, String schemaName, String categoryName, String[] subCategories) {
		Category category = addCategory(asset, schemaName, categoryName);
		SubCategory subCategory = null;
		StringBuffer fullPath = new StringBuffer(schemaName + AssetConstants.FILE_DELIM + categoryName);
		for (int i = 0; i < subCategories.length; i++) {
			fullPath.append(AssetConstants.FILE_DELIM + subCategories[i]);
			SubCategory parent = subCategory;
			SubCategory[] oldSubCategories = null;
			if (parent != null) {
				subCategory = parent.getSubCategory(subCategories[i]);
				oldSubCategories = parent.getSubCategories();
			} else {
				subCategory = category.getSubCategory(subCategories[i]);
				oldSubCategories = category.getSubCategories();
			}
			if (subCategory == null) {
				if (oldSubCategories == null)
					oldSubCategories = new SubCategory[0];
				SubCategory[] newSubCategories = new SubCategory[oldSubCategories.length + 1];
				System.arraycopy(oldSubCategories, 0, newSubCategories, 0, oldSubCategories.length);
				subCategory = new SubCategory();
				subCategory.setName(subCategories[i]);
				subCategory.setFullPath(fullPath.toString());
				newSubCategories[newSubCategories.length - 1] = subCategory;
				if (parent != null)
					parent.setSubCategories(newSubCategories);
				else
					category.setSubCategories(newSubCategories);
			}
		}
		if (subCategory != null)
			subCategory.setSet(true);
		return subCategory;
	}
	
	private static Category addCategory(Asset asset, String schemaName, String categoryName) {
		CategorySchema schema = addCategorySchema(asset, schemaName);
		Category category = schema.getCategory(categoryName);
		if (category == null) {
			Category[] oldCategories = schema.getCategories();
			if (oldCategories == null) 
				oldCategories = new Category[0];
			Category[] newCategories = new Category[oldCategories.length + 1];
			System.arraycopy(oldCategories, 0, newCategories, 0, oldCategories.length);
			category = new Category();
			category.setName(categoryName);
			newCategories[newCategories.length - 1] = category;
			schema.setCategories(newCategories);
		}
		return category;
	}
	
	private static CategorySchema addCategorySchema(Asset asset, String schemaName) {
		CategorySchema schema = asset.getCategorySchema(schemaName);
		if (schema == null) {
			CategorySchema[] oldSchemas = asset.getCategorySchemas();
			if (oldSchemas == null)
				oldSchemas = new CategorySchema[0];
			CategorySchema[] newSchemas = new CategorySchema[oldSchemas.length + 1];
			System.arraycopy(oldSchemas, 0, newSchemas, 0, oldSchemas.length);
			schema = new CategorySchema();
			schema.setName(schemaName);
			newSchemas[newSchemas.length - 1] = schema;
			asset.setCategorySchemas(newSchemas);
		}
		return schema;
	}
	
	private static void processRelatedAssets(ManifestAccessor manifestAccessor, Asset asset) {
		Iterator relatedAssets = manifestAccessor.getRelatedAssets().iterator();
		List rels = new ArrayList();
		while (relatedAssets.hasNext()) {
			RelatedAsset relatedAsset = (RelatedAsset)relatedAssets.next();
			Relationship rel = new Relationship();
			rel.setParentAsset(asset);
			rel.setChildAssetGUID(relatedAsset.getAssetId());
			rel.setChildAssetVersion(relatedAsset.getAssetVersion());
			rel.setRelationshipTypeName(relatedAsset.getRelationshipType());
			rels.add(rel);
		}
		asset.setRelationships((Relationship[])rels.toArray(new Relationship[rels.size()]));
	}
	
	private static void processTimes(ManifestAccessor manifestAccessor, Asset asset) {
		String[] creationTime = manifestAccessor.getCreationTime();
		if (creationTime != null) {
			PersonTime pt = new PersonTime();
			pt.setAmount(Double.parseDouble(creationTime[0]));
			pt.setUnit(creationTime[1]);
			asset.setCreationTime(pt);
		}
		String[] usageTime = manifestAccessor.getUsageTime();
		if (usageTime != null) {
			PersonTime pt = new PersonTime();
			pt.setAmount(Double.parseDouble(usageTime[0]));
			pt.setUnit(usageTime[1]);
			asset.setUsageTime(pt);
		}
	}
	
	private static String computePath(FolderArtifact parent) {
		if (parent == null || parent.getPath() == null)
			return ""; //$NON-NLS-1$
		if (parent.getPath().length() == 0)
			return parent.getName();
		return parent.getPath() + "/" + parent.getName(); //$NON-NLS-1$
	}
	
	private static void processArtifacts(ManifestAccessor manifestAccessor, List artifactList, FolderArtifact folderArtifact, Asset asset, File artifactRoot) {
		Iterator artifacts = artifactList.iterator();
		List arts = new ArrayList();
		while (artifacts.hasNext()) {
			com.ibm.ram.defaultprofile.Artifact artifact = (com.ibm.ram.defaultprofile.Artifact)artifacts.next();
			if (ManifestAccessor.FOLDER_ARTIFACT_TYPE.equals(artifact.getType())) {
				FolderArtifact fa = new FolderArtifact();
				fa.setName(artifact.getName());		
				fa.setPath(computePath(folderArtifact));
				arts.add(fa);
				processArtifacts(manifestAccessor, artifact.getArtifact(), fa, asset, artifactRoot);
			} else if (ManifestAccessor.URL_ARTIFACT_TYPE.equals(artifact.getType())) {
				URLArtifact urlArtifact = new URLArtifact();
				urlArtifact.setName(artifact.getName());
				urlArtifact.setPath(computePath(folderArtifact));
				Reference ref = artifact.getReference();
				if (ref != null)
					urlArtifact.setURL(ref.getValue());
				arts.add(urlArtifact);
			} else { // file artifact
				String filePath = artifact.computePathInSolution(AssetConstants.FILE_DELIM);
				File localFile = new File(artifactRoot, filePath);
				LocalFileArtifact fileArtifact = new LocalFileArtifact(localFile);
				fileArtifact.setPath(computePath(folderArtifact));
				String label = manifestAccessor.getLabel(artifact);
				if (label != null && label.length() > 0)
					fileArtifact.setLabel(label);
				arts.add(fileArtifact);
			}
		}
		folderArtifact.setChildren((Artifact[])arts.toArray(new Artifact[arts.size()]));
	}
	
	private static void processAssetType(ManifestAccessor manifestAccessor, Asset asset) {
		AssetType assetType = new AssetType();
		try {
			NodeDescriptor descriptor = (NodeDescriptor)Utilities.loadClassificationSchemaObjectFromURI(
					Utilities.createClassificationSchemaURI(manifestAccessor.getAssetTypeLocalized(), manifestAccessor.getManifest()), manifestAccessor.getManifest());
			NodeDescriptor parent = descriptor.getGeneral();
			while (parent != null) {
				NodeDescriptor temp = parent;
				parent = parent.getGeneral();
				if (parent != null)
					descriptor = temp;
			}
			if (descriptor != null && descriptor.getName() != null) {
				assetType.setName(descriptor.getName());
				asset.setAssetType(assetType);
				return;
			}
		} catch (Exception e) {}
		
		// getting the actual asset type from the URI didn't work, so create a fake asset type based on the URI
		assetType.setURI(manifestAccessor.getAssetTypeURI().toString());
		assetType.setName(createNameFromURI(manifestAccessor.getAssetTypeURI().toString()));
		asset.setAssetType(assetType);
	}
	
	private static String createNameFromURI(String uri) {
		if (uri == null)
			return null;
		String name = uri;
		int i = name.lastIndexOf("#"); //$NON-NLS-1$
		if (i > -1 && i < name.length() - 1)
			name = name.substring(i + 1);
		if (name.equals("/")) { //$NON-NLS-1$
			name = uri.substring(0, i);
			i = name.lastIndexOf('/');
			if (i > -1 && i < name.length() - 1) {
				int j = name.lastIndexOf('.');
				if (j > i + 1)
					name = name.substring(i + 1, j);
				else
					name = name.substring(i + 1);
			}
		}
		return name;
	}
	
	private static void processAttributes(ManifestAccessor manifestAccessor, Asset asset) {
		Map attributeMap = manifestAccessor.initializeCustomAttributes(true, Collections.EMPTY_LIST);
		AssetAttribute[] newAttributes = new AssetAttribute[attributeMap.size()];
		Iterator attributes = attributeMap.values().iterator();
		int index = 0;
		while (attributes.hasNext()) {
			ManifestAccessor.CustomAttribute attribute = (ManifestAccessor.CustomAttribute)attributes.next();
			AssetAttribute newAttribute = new AssetAttribute();
			try {
				newAttribute.setName(attribute.getName());
			} catch (Exception e) {
				newAttribute.setName(createNameFromURI(attribute.getURIString()));
			}
			if (attribute.isRestrictedValues()) {
				NodeDescriptor[] valueDescriptors = attribute.getRestrictedAttributeValues();
				String[] values = new String[valueDescriptors.length];
				for (int i = 0; i < values.length; i++) {
					values[i] = valueDescriptors[i].getName();
				}
				newAttribute.setValues(values);
			} else {
				newAttribute.setValues(new String[] {attribute.getUnrestrictedAttributeValue()});
			}
			newAttributes[index++] = newAttribute;
		}
		asset.setAssetAttributes(newAttributes);
	}
	
	private static void processCategories(ManifestAccessor manifestAccessor, Asset asset) {
		Map classificationInfoMap = manifestAccessor.initializeClassifications(true, Collections.EMPTY_LIST);
		Iterator classificationInfos = classificationInfoMap.values().iterator();
		while (classificationInfos.hasNext()) {
			ClassificationInfo info = (ClassificationInfo)classificationInfos.next();
			Iterator descriptors = info.getNodeDescriptors().iterator();
			while (descriptors.hasNext()) {
				NodeDescriptor nodeDescriptor = (NodeDescriptor)descriptors.next();
				if (nodeDescriptor.getName() != null) {
					StringBuffer buffer = new StringBuffer(nodeDescriptor.getName());
					NodeDescriptor parent = nodeDescriptor.getGeneral();
					while (parent != null) {
						buffer.insert(0, parent.getName() + "/"); //$NON-NLS-1$
						parent = parent.getGeneral();
					}
					addSubCategory(asset, info.getSchema().getName() + "/" + buffer.toString()); //$NON-NLS-1$
				} else {
					URI categoryUri = EcoreUtil.getURI(nodeDescriptor);
					URI schemaUri = EcoreUtil.getURI(info.getSchema());
					if (categoryUri != null && schemaUri != null) {
						String subCategoryName = createNameFromURI(categoryUri.toString());
						String schemaName = createNameFromURI(schemaUri.toString());
						addSubCategory(asset, schemaName + "/" + Messages.Batch_FileSystem_UnknownCategory + "/" + subCategoryName); //$NON-NLS-1$//$NON-NLS-2$
					}
				}
			}
		}
	}
	
	public static void processManifest(File manifestFile, Asset asset) {
		ManifestAccessor manifestAccessor = loadManifest(manifestFile);

		asset.setName(manifestAccessor.getName());
		asset.getIdentification().setVersion(manifestAccessor.getVersion());
		asset.getIdentification().setGUID(manifestAccessor.getId());
		asset.setShortDescription(manifestAccessor.getShortDescription());
		asset.setDescription(manifestAccessor.getDescription());

		processAssetType(manifestAccessor, asset);
		processTimes(manifestAccessor, asset);
		processAttributes(manifestAccessor, asset);
		processRelatedAssets(manifestAccessor, asset);
		
		Solution solution = manifestAccessor.getSolution();
		if (solution != null) {
			FolderArtifact root = asset.getArtifactsRoot();
			if (root == null) {
				root = new FolderArtifact();
				asset.setArtifactsRoot(root);
			}
			processArtifacts(manifestAccessor, solution.getArtifact(), root, asset, manifestFile.getParentFile());
		}
		processCategories(manifestAccessor, asset);
	}
	
}
